@@ -1,4 +1,4 @@
-__version__ = '0.19.3'
+__version__ = '1.2.4'
 
 from utilix import uconfig
 from .common import *
@@ -16,6 +16,7 @@ from .rundb import *
 from .scada import *
 from .bokeh_utils import *
 from .rucio import *
+from .url_config import *
 
 from . import plugins
 from .plugins import *
@@ -26,6 +27,9 @@ from . import analyses
 # Otherwise we have straxen.demo() etc.
 from . import contexts
 
+from . import test_utils
+from .test_utils import *
+
 try:
     from . import holoviews_utils
     from .holoviews_utils import *
