@@ -2,16 +2,13 @@ import ast
 import configparser
 import gzip
 import inspect
-import io
+import typing as ty
 import commentjson
 import json
 import os
 import os.path as osp
 import pickle
 import dill
-import socket
-import sys
-import tarfile
 import urllib.request
 import tqdm
 import numpy as np
@@ -25,7 +22,8 @@ import straxen
 export, __all__ = strax.exporter()
 __all__ += ['straxen_dir', 'first_sr1_run', 'tpc_r', 'tpc_z', 'aux_repo',
             'n_tpc_pmts', 'n_top_pmts', 'n_hard_aqmon_start', 'ADC_TO_E',
-            'n_nveto_pmts', 'n_mveto_pmts', 'tpc_pmt_radius', 'cryostat_outer_radius']
+            'n_nveto_pmts', 'n_mveto_pmts', 'tpc_pmt_radius', 'cryostat_outer_radius',
+            'perp_wire_angle', 'perp_wire_x_rot_pos', 'INFINITY_64BIT_SIGNED']
 
 straxen_dir = os.path.dirname(os.path.abspath(
     inspect.getfile(inspect.currentframe())))
@@ -44,6 +42,9 @@ n_mveto_pmts = 84
 
 tpc_pmt_radius = 7.62 / 2  # cm
 
+perp_wire_angle = np.deg2rad(30)
+perp_wire_x_rot_pos = 13.06  #[cm]
+
 # Convert from ADC * samples to electrons emitted by PMT
 # see pax.dsputils.adc_to_pe for calculation. Saving this number in straxen as
 # it's needed in analyses
@@ -53,6 +54,32 @@ ADC_TO_E = 17142.81741
 LAST_MISCABLED_RUN = 8796
 TSTART_FIRST_CORRECTLY_CABLED_RUN = 1596036001000000000
 
+INFINITY_64BIT_SIGNED = 9223372036854775807
+
+
+@export
+def rotate_perp_wires(x_obs: np.ndarray,
+                      y_obs: np.ndarray,
+                      angle_extra: ty.Union[float, int] = 0):
+    """
+    Returns x and y in the rotated plane where the perpendicular wires
+    area vertically aligned (parallel to the y-axis). Accepts addition to the
+    rotation angle with `angle_extra` [deg]
+
+    :param x_obs: array of x coordinates
+    :param y_obs: array of y coordinates
+    :param angle_extra: extra rotation in [deg]
+    :return: x_rotated, y_rotated
+    """
+    if len(x_obs) != len(y_obs):
+        raise ValueError('x and y are not of the same length')
+    angle_extra_rad = np.deg2rad(angle_extra)
+    x_rot = (np.cos(perp_wire_angle + angle_extra_rad) * x_obs
+             - np.sin(perp_wire_angle + angle_extra_rad) * y_obs)
+    y_rot = (np.sin(perp_wire_angle + angle_extra_rad) * x_obs
+             + np.cos(perp_wire_angle + angle_extra_rad) * y_obs)
+    return x_rot, y_rot
+
 
 @export
 def pmt_positions(xenon1t=False):
@@ -92,9 +119,10 @@ def open_resource(file_name: str, fmt='text'):
     :param fmt: format of the file
     :return: opened file
     """
-    if file_name in _resource_cache:
+    cached_name = _cache_name(file_name, fmt)
+    if cached_name in _resource_cache:
         # Retrieve from in-memory cache
-        return _resource_cache[file_name]
+        return _resource_cache[cached_name]
     # File resource
     if fmt in ['npy', 'npy_pickle']:
         result = np.load(file_name, allow_pickle=fmt == 'npy_pickle')
@@ -134,7 +162,7 @@ def open_resource(file_name: str, fmt='text'):
         raise ValueError(f"Unsupported format {fmt}!")
 
     # Store in in-memory cache
-    _resource_cache[file_name] = result
+    _resource_cache[cached_name] = result
 
     return result
 
@@ -159,8 +187,9 @@ def get_resource(x: str, fmt='text'):
         specified format
     """
     # 1. load from memory
-    if x in _resource_cache:
-        return _resource_cache[x]
+    cached_name = _cache_name(x, fmt)
+    if cached_name in _resource_cache:
+        return _resource_cache[cached_name]
     # 2. load from file
     elif os.path.exists(x):
         return open_resource(x, fmt=fmt)
@@ -178,6 +207,11 @@ def get_resource(x: str, fmt='text'):
         f'cannot download it from anywhere.')
 
 
+def _cache_name(name: str, fmt: str)->str:
+    """Return a name under which to store the requested name with the given format in the _cache"""
+    return f'{fmt}::{name}'
+
+
 # Legacy loader for public URL files
 def resource_from_url(html: str, fmt='text'):
     """
@@ -241,70 +275,6 @@ def resource_from_url(html: str, fmt='text'):
     return result
 
 
-@export
-def get_secret(x):
-    """Return secret key x. In order of priority, we search:
-      * Environment variable: uppercase version of x
-      * xenon_secrets.py (if included with your straxen installation)
-      * A standard xenon_secrets.py located on the midway analysis hub
-        (if you are running on midway)
-    """
-    warn("xenon_secrets is deprecated, and will be replaced with utilix"
-         "configuration file instead. See https://github.com/XENONnT/utilix")
-    env_name = x.upper()
-    if env_name in os.environ:
-        return os.environ[env_name]
-
-    message = (f"Secret {x} requested, but there is no environment "
-               f"variable {env_name}, ")
-
-    # now try using utilix. We need to check that it is not None first!
-    # this will be main method in a future release
-    if straxen.uconfig is not None and straxen.uconfig.has_option('straxen', x):
-        try:
-            return straxen.uconfig.get('straxen', x)
-        except configparser.NoOptionError:
-            warn(f'straxen.uconfig does not have {x}')
-
-    # if that doesn't work, revert to xenon_secrets
-    try:
-        from . import xenon_secrets
-    except ImportError:
-        message += ("nor was there a valid xenon_secrets.py "
-                    "included with your straxen installation, ")
-
-        # If on midway, try loading a standard secrets file instead
-        if 'rcc' in socket.getfqdn():
-            path_to_secrets = '/project2/lgrandi/xenonnt/xenon_secrets.py'
-            if os.path.exists(path_to_secrets):
-                sys.path.append(osp.dirname(path_to_secrets))
-                import xenon_secrets
-                sys.path.pop()
-            else:
-                raise ValueError(
-                    message + ' nor could we load the secrets module from '
-                              f'{path_to_secrets}, even though you seem '
-                              'to be on the midway analysis hub.')
-
-        else:
-            raise ValueError(
-                message + 'nor are you on the midway analysis hub.')
-
-    if hasattr(xenon_secrets, x):
-        return getattr(xenon_secrets, x)
-    raise ValueError(message + " and the secret is not in xenon_secrets.py")
-
-
-@export
-def download_test_data():
-    """Downloads strax test data to strax_test_data in the current directory"""
-    blob = get_resource('https://raw.githubusercontent.com/XENONnT/strax_auxiliary_files/11929e7595e178dd335d59727024847efde530fb/strax_test_data_straxv0.9.tar',
-                        fmt='binary')
-    f = io.BytesIO(blob)
-    tf = tarfile.open(fileobj=f)
-    tf.extractall()
-
-
 @export
 def get_livetime_sec(context, run_id, things=None):
     """Get the livetime of a run in seconds. If it is not in the run metadata,
@@ -340,7 +310,7 @@ def pre_apply_function(data, run_id, target, function_name='pre_apply_function')
     if function_name not in _resource_cache:
         # only load the function once and put it in the resource cache
         function_file = f'{function_name}.py'
-        function_file = _overwrite_testing_function_file(function_file)
+        function_file = straxen.test_utils._overwrite_testing_function_file(function_file)
         function = get_resource(function_file, fmt='txt')
         # pylint: disable=exec-used
         exec(function)
@@ -350,32 +320,9 @@ def pre_apply_function(data, run_id, target, function_name='pre_apply_function')
     return data
 
 
-def _overwrite_testing_function_file(function_file):
-    """For testing purposes allow this function file to be loaded from HOME/testing_folder"""
-    if not straxen._is_on_pytest():
-        # If we are not on a pytest, never try using a local file.
-        return function_file
-
-    home = os.environ.get('HOME')
-    if home is None:
-        # Impossible to load from non-existent folder
-        return function_file
-
-    testing_file = os.path.join(home, function_file)
-
-    if os.path.exists(testing_file):
-        # For testing purposes allow loading from 'home/testing_folder'
-        warn(f'Using local function: {function_file} from {testing_file}! '
-             f'If you are not integrated testing on github you should '
-             f'absolutely remove this file. (See #559)')
-        function_file = testing_file
-
-    return function_file
-
-
 @export
 def check_loading_allowed(data, run_id, target,
-                          max_in_disallowed = 1,
+                          max_in_disallowed=1,
                           disallowed=('event_positions',
                                       'corrected_areas',
                                       'energy_estimates')
@@ -429,16 +376,6 @@ def remap_channels(data, verbose=True, safe_copy=False, _tqdm=False, ):
         aux_repo + '/ecb6da7bd4deb98cd0a4e83b3da81c1e67505b16/remapped_channels_since_20200729_17.20UTC.csv',
         fmt='csv')
 
-    def wr_tqdm(x):
-        """Wrap input x with tqdm"""
-        if _tqdm:
-            try:
-                return tqdm.tqdm_notebook(x)
-            except (AttributeError, ModuleNotFoundError, ImportError):
-                # ok, sorry lets not wrap but return x
-                pass
-        return x
-
     def convert_channel(_data, replace=('channel', 'max_pmt')):
         """
         Given an array, replace the 'channel' entry if we had to remap it according to the
@@ -518,7 +455,7 @@ def remap_channels(data, verbose=True, safe_copy=False, _tqdm=False, ):
             return channel_data
         # Create a buffer to overright
         buffer = channel_data.copy()
-        for k in wr_tqdm(get_dtypes(channel_data)):
+        for k in strax.utils.tqdm(get_dtypes(channel_data), disable=not _tqdm):
             if np.iterable(channel_data[k][0]) and len(channel_data[k][0]) == n_chs:
                 if verbose:
                     print(f'convert_channel_like::\tupdate {k}')
@@ -613,6 +550,7 @@ def _swap_values_in_array(data_arr, buffer, items, replacements):
                 break
     return buffer
 
+
 ##
 # Old XENON1T Stuff
 ##
