@@ -2,10 +2,13 @@ from immutabledict import immutabledict
 import strax
 import straxen
 from copy import deepcopy
+from .rucio import HAVE_ADMIX
+import os
+
+from straxen.common import pax_file
 
 common_opts = dict(
     register_all=[
-        straxen.event_processing,
         straxen.double_scatter,
     ],
     # Register all peak/pulse processing by hand as 1T does not need to have
@@ -17,11 +20,17 @@ common_opts = dict(
         straxen.MergedS2s,
         straxen.Peaks,
         straxen.PeakBasics,
-        straxen.PeakProximity],
+        straxen.PeakProximity,
+        straxen.Events,
+        straxen.EventBasics,
+        straxen.EventPositions,
+        straxen.CorrectedAreas,
+        straxen.EnergyEstimates,
+    ],
     check_available=('raw_records', 'peak_basics'),
     store_run_fields=(
         'name', 'number',
-        'start', 'end', 'livetime', 'mode'))
+        'start', 'end', 'livetime', 'mode', 'source'))
 
 xnt_common_config = dict(
     n_tpc_pmts=straxen.n_tpc_pmts,
@@ -44,11 +53,7 @@ xnt_common_config = dict(
         nveto_blank=(2999, 2999)),
     # Clustering/classification parameters
     # Event level parameters
-    s2_xy_correction_map=('s2_xy_map', "ONLINE", True),
     fdc_map=('fdc_map', "ONLINE", True),
-    s1_xyz_correction_map=("s1_xyz_map", "ONLINE", True),
-    g1=0.1426,
-    g2=11.55,
 )
 # these are placeholders to avoid calling cmt with non integer run_ids. Better solution pending.
 # s1,s2 and fd corrections are still problematic
@@ -56,7 +61,7 @@ xnt_simulation_config = deepcopy(xnt_common_config)
 xnt_simulation_config.update(gain_model=("to_pe_placeholder", True),
                              gain_model_nv=("adc_nv", True),
                              gain_model_mv=("adc_mv", True),
-                             elife_conf=('elife_constant', 1e6),
+                             elife=1e6,
                              )
 
 # Plugins in these files have nT plugins, E.g. in pulse&peak(let)
@@ -68,13 +73,17 @@ xnt_common_opts.update({
                                            straxen.PeakPositionsMLP,
                                            straxen.PeakPositionsGCN,
                                            straxen.PeakPositionsNT,
+                                           straxen.S2ReconPosDiff,
                                            straxen.PeakBasicsHighEnergy,
                                            straxen.PeaksHighEnergy,
                                            straxen.PeakletsHighEnergy,
                                            straxen.PeakletClassificationHighEnergy,
                                            straxen.MergedS2sHighEnergy,
+                                           straxen.PeakVetoTagging,
                                            straxen.EventInfo,
-                                          ],
+                                           straxen.PeakShadow,
+                                           straxen.EventShadow,
+                                           ],
     'register_all': common_opts['register_all'] + [straxen.veto_veto_regions,
                                                    straxen.nveto_recorder,
                                                    straxen.veto_pulse_processing,
@@ -86,10 +95,10 @@ xnt_common_opts.update({
                                                    straxen.online_monitor,
                                                    straxen.event_area_per_channel,
                                                    straxen.event_patternfit,
+                                                   straxen.event_processing,
                                                    ],
     'use_per_run_defaults': False,
 })
-
 ##
 # XENONnT
 ##
@@ -103,14 +112,14 @@ def xenonnt(cmt_version='global_ONLINE', **kwargs):
 
 
 def xenonnt_online(output_folder='./strax_data',
-                   use_rucio=False,
                    we_are_the_daq=False,
+                   download_heavy=False,
                    _minimum_run_number=7157,
                    _maximum_run_number=None,
                    _database_init=True,
                    _forbid_creation_of=None,
-                   _rucio_path='/dali/lgrandi/rucio/',
                    _include_rucio_remote=False,
+                   _rucio_path='/dali/lgrandi/rucio/',
                    _raw_path='/dali/lgrandi/xenonnt/raw',
                    _processed_path='/dali/lgrandi/xenonnt/processed',
                    _add_online_monitor_frontend=False,
@@ -121,8 +130,8 @@ def xenonnt_online(output_folder='./strax_data',
 
     :param output_folder: str, Path of the strax.DataDirectory where new
         data can be stored
-    :param use_rucio: bool, whether or not to use the rucio frontend
     :param we_are_the_daq: bool, if we have admin access to upload data
+    :param download_heavy: bool, whether or not to allow downloads of heavy data (raw_records*, less the aqmon)
     :param _minimum_run_number: int, lowest number to consider
     :param _maximum_run_number: Highest number to consider. When None
         (the default) consider all runs that are higher than the
@@ -147,7 +156,7 @@ def xenonnt_online(output_folder='./strax_data',
     st = strax.Context(
         config=straxen.contexts.xnt_common_config,
         **context_options)
-    st.register([straxen.DAQReader, straxen.LEDCalibration])
+    st.register([straxen.DAQReader, straxen.LEDCalibration, straxen.LEDAfterpulseProcessing])
 
     st.storage = [
         straxen.RunDB(
@@ -169,21 +178,21 @@ def xenonnt_online(output_folder='./strax_data',
                 readonly=True,
             )]
         if output_folder:
-            st.storage.append(
-                strax.DataDirectory(output_folder,
-                                    provide_run_metadata=True,
-                                   ))
-
+            st.storage += [strax.DataDirectory(output_folder,
+                                               provide_run_metadata=True,
+                                               )]
         st.context_config['forbid_creation_of'] = straxen.daqreader.DAQReader.provides
         if _forbid_creation_of is not None:
             st.context_config['forbid_creation_of'] += strax.to_str_tuple(_forbid_creation_of)
 
-    # if we said so, add the rucio frontend to storage
-    if use_rucio:
-        st.storage.append(straxen.rucio.RucioFrontend(
-            include_remote=straxen.RUCIO_AVAILABLE,
-            staging_dir=output_folder,
-        ))
+    # Add the rucio frontend if we are able to
+    if HAVE_ADMIX:
+        rucio_frontend = straxen.rucio.RucioFrontend(
+            include_remote=_include_rucio_remote,
+            staging_dir=os.path.join(output_folder, 'rucio'),
+            download_heavy=download_heavy,
+        )
+        st.storage += [rucio_frontend]
 
     # Only the online monitor backend for the DAQ
     if _database_init and (_add_online_monitor_frontend or we_are_the_daq):
@@ -191,7 +200,10 @@ def xenonnt_online(output_folder='./strax_data',
             readonly=not we_are_the_daq,
             take_only=('veto_intervals',
                        'online_peak_monitor',
-                       'event_basics',))]
+                       'event_basics',
+                       'online_monitor_nv',
+                       'online_monitor_mv',
+                       ))]
 
     # Remap the data if it is before channel swap (because of wrongly cabled
     # signal cable connectors) These are runs older than run 8797. Runs
@@ -209,42 +221,80 @@ def xenonnt_online(output_folder='./strax_data',
 
 def xenonnt_led(**kwargs):
     st = xenonnt_online(**kwargs)
-    st.context_config['check_available'] = ('raw_records', 'led_calibration')
+    st.set_context_config(
+        {'check_available': ('raw_records', 'led_calibration'),
+         'free_options': list(xnt_common_config.keys())
+         })
     # Return a new context with only raw_records and led_calibration registered
     st = st.new_context(
         replace=True,
         config=st.config,
         storage=st.storage,
         **st.context_config)
-    st.register([straxen.DAQReader, straxen.LEDCalibration])
+    st.register([straxen.DAQReader, 
+                 straxen.LEDCalibration,
+                 straxen.nVETORecorder,
+                 straxen.nVETOPulseProcessing,
+                 straxen.nVETOHitlets,
+                 straxen.nVetoExtTimings, ])
+    st.set_config({"coincidence_level_recorder_nv": 1})
     return st
 
 
-def xenonnt_simulation(output_folder='./strax_data',
-                       cmt_run_id='020280',
-                       cmt_version='v3',
-                       fax_config='fax_config_nt_design.json',
-                       cmt_option_overwrite=immutabledict(),
-                       overwrite_from_fax_file=False,
-                       _cmt_run_id_proc_only=None,
-                       _forbid_creation_of=None,
-                       _config_overlap=immutabledict(drift_time_gate='electron_drift_time_gate',
-                                                     drift_velocity_liquid='electron_drift_velocity',
-                                                     electron_lifetime_liquid='elife_conf'),
-                       **kwargs):
+def xenonnt_simulation(
+                output_folder='./strax_data',
+                wfsim_registry='RawRecordsFromFaxNT',
+                cmt_run_id_sim=None,
+                cmt_run_id_proc=None,
+                cmt_version='global_ONLINE',
+                fax_config='fax_config_nt_design.json',
+                overwrite_from_fax_file_sim=False,
+                overwrite_from_fax_file_proc=False,
+                cmt_option_overwrite_sim=immutabledict(),
+                cmt_option_overwrite_proc=immutabledict(),
+                _forbid_creation_of=None,
+                _config_overlap=immutabledict(
+                            drift_time_gate='electron_drift_time_gate',
+                            drift_velocity_liquid='electron_drift_velocity',
+                            electron_lifetime_liquid='elife',
+                ),
+                **kwargs):
     """
-    Context with CMT options updated with cmt_run_id.
+    The most generic context that allows for setting full divergent
+    settings for simulation purposes
+
+    It makes full divergent setup, allowing to set detector simulation
+    part (i.e. for wfsim up to truth and  raw_records). Parameters _sim
+    refer to detector simulation parameters.
+
+    Arguments having _proc in their name refer to detector parameters that
+    are used for processing of simulations as done to the real detector
+    data. This means starting from already existing raw_records and finishing
+    with higher level data, such as peaks, events etc.
+
+    If only one cmt_run_id is given, the second one will be set automatically,
+    resulting in CMT match between simulation and processing. However, detector
+    parameters can be still overwritten from fax file or manually using cmt
+    config overwrite options.
+
     CMT options can also be overwritten via fax config file.
     :param output_folder: Output folder for strax data.
-    :param cmt_run_id: Run id which should be used to load corrections.
+    :param wfsim_registry: Name of WFSim plugin used to generate data.
+    :param cmt_run_id_sim: Run id for detector parameters from CMT to be used
+        for creation of raw_records.
+    :param cmt_run_id_proc: Run id for detector parameters from CMT to be used
+        for processing from raw_records to higher level data.
     :param cmt_version: Global version for corrections to be loaded.
     :param fax_config: Fax config file to use.
-    :param cmt_option_overwrite: Dictionary to overwrite CMT settings. Keys
-        must be valid CMT option keys.
-    :param overwrite_from_fax_file: If true overwrites CMT options with
-        constants from fax file.
-    :param _cmt_run_id_proc_only: Run id just for > raw_records processing if diverge from
-    cmt_run_id.
+    :param overwrite_from_fax_file_sim: If true sets detector simulation
+        parameters for truth/raw_records from from fax_config file istead of CMT
+    :param overwrite_from_fax_file_proc:  If true sets detector processing
+        parameters after raw_records(peaklets/events/etc) from from fax_config
+        file instead of CMT
+    :param cmt_option_overwrite_sim: Dictionary to overwrite CMT settings for
+        the detector simulation part.
+    :param cmt_option_overwrite_proc: Dictionary to overwrite CMT settings for
+        the data processing part.
     :param _forbid_creation_of: str/tuple, of datatypes to prevent form
         being written (e.g. 'raw_records' for read only simulation context).
     :param _config_overlap: Dictionary of options to overwrite. Keys
@@ -260,64 +310,132 @@ def xenonnt_simulation(output_folder='./strax_data',
                     check_raw_record_overlaps=True,
                     **straxen.contexts.xnt_common_config,),
         **straxen.contexts.xnt_common_opts, **kwargs)
-    st.register(wfsim.RawRecordsFromFaxNT)
+    st.register(getattr(wfsim, wfsim_registry))
+
+    # Make sure that the non-simulated raw-record types are not requested
+    st.deregister_plugins_with_missing_dependencies()
+
+    if straxen.utilix_is_configured(
+            warning_message='Bad context as we cannot set CMT since we '
+                            'have no database access'''):
+        st.apply_cmt_version(cmt_version)
 
     if _forbid_creation_of is not None:
         st.context_config['forbid_creation_of'] += strax.to_str_tuple(_forbid_creation_of)
 
-    st.apply_cmt_version(f'global_{cmt_version}')
-    if not cmt_run_id:
-        raise ValueError('You have to specify a run_id which should be used to initialize '
-                         'the corrections.')
-    if _cmt_run_id_proc_only is None:
-        _cmt_run_id_proc_only = cmt_run_id
-
-    # Setup processing
-    # Replace default cmt options with cmt_run_id tag + cmt run id
-    cmt_options = straxen.get_corrections.get_cmt_options(st)
-    for option in cmt_options:
-        st.config[option] = tuple(['cmt_run_id', _cmt_run_id_proc_only, *cmt_options[option]])
-
-    # Take fax config and put into context option
-    if overwrite_from_fax_file:
-        fax_config = straxen.get_resource(fax_config, fmt='json')
-
-        for fax_field, cmt_field in _config_overlap.items():
-            st.config[cmt_field] = tuple([cmt_options[cmt_field][0] + '_constant',
-                                          fax_config[fax_field]])
-
-    # Setup simulation
-    # Pass the CMT options to simulation (override all other config input methods)
+    # doing sanity checks for cmt run ids for simulation and processing
+    if (not cmt_run_id_sim ) and (not cmt_run_id_proc ):
+        raise RuntimeError("cmt_run_id_sim and cmt_run_id_proc are None. "
+                           "You have to specify at least one CMT run id. ")
+    if (cmt_run_id_sim and cmt_run_id_proc ) and (cmt_run_id_sim!=cmt_run_id_proc):
+        print("INFO : divergent CMT runs for simulation and processing")
+        print("    cmt_run_id_sim".ljust(25), cmt_run_id_sim)
+        print("    cmt_run_id_proc".ljust(25), cmt_run_id_proc)
     else:
-        fax_config_override_from_cmt = dict()
+        cmt_id = cmt_run_id_sim  or cmt_run_id_proc
+        cmt_run_id_sim  = cmt_id
+        cmt_run_id_proc = cmt_id
 
+    # Replace default cmt options with cmt_run_id tag + cmt run id
+    cmt_options_full = straxen.get_corrections.get_cmt_options(st)
+
+    # prune to just get the strax options
+    cmt_options = {key: val['strax_option']
+                   for key, val in cmt_options_full.items()}
+
+    # First, fix gain model for simulation
+    st.set_config({'gain_model_mc': 
+                        ('cmt_run_id', cmt_run_id_sim, *cmt_options['gain_model'])})
+    fax_config_override_from_cmt = dict()
+    for fax_field, cmt_field in _config_overlap.items():
+        value = cmt_options[cmt_field]
+
+        # URL configs need to be converted to the expected format
+        if isinstance(value, str):
+            opt_cfg = cmt_options_full[cmt_field]
+            version = straxen.URLConfig.kwarg_from_url(value, 'version')
+            # We now allow the cmt name to be different from the config name
+            # WFSim expects the cmt name
+            value = (opt_cfg['correction'], version, True)
+
+        fax_config_override_from_cmt[fax_field] = ('cmt_run_id', cmt_run_id_sim,
+                                                   *value)
+    st.set_config({'fax_config_override_from_cmt': fax_config_override_from_cmt})
+
+    # and all other parameters for processing
+    for option in cmt_options:
+        value = cmt_options[option]
+        if isinstance(value, str):
+            # for URL configs we can just replace the run_id keyword argument
+            # This will become the proper way to override the run_id for cmt configs
+            st.config[option] = straxen.URLConfig.format_url_kwargs(value, run_id=cmt_run_id_proc)
+        else:
+            # FIXME: Remove once all cmt configs are URLConfigs
+            st.config[option] = ('cmt_run_id', cmt_run_id_proc, *value)
+
+    # Done with "default" usage, now to overwrites from file
+    #
+    # Take fax config and put into context option
+    if overwrite_from_fax_file_proc or overwrite_from_fax_file_sim:
+        fax_config = straxen.get_resource(fax_config, fmt='json')
         for fax_field, cmt_field in _config_overlap.items():
-            fax_config_override_from_cmt[fax_field] = tuple(['cmt_run_id', cmt_run_id, *cmt_options[cmt_field]])
-        st.set_config({'fax_config_override_from_cmt': fax_config_override_from_cmt})
+            if overwrite_from_fax_file_proc:
+                if isinstance(cmt_options[cmt_field], str):
+                    # URLConfigs can just be set to a constant
+                    st.config[cmt_field] = fax_config[fax_field]
+                else:
+                    # FIXME: Remove once all cmt configs are URLConfigs
+                    st.config[cmt_field] = (cmt_options[cmt_field][0] + '_constant',
+                                            fax_config[fax_field])
+            if overwrite_from_fax_file_sim:
+                # CMT name allowed to be different from the config name
+                # WFSim needs the cmt name
+                cmt_name = cmt_options_full[cmt_field]['correction']
 
-    # Fix gain model
-    st.set_config({'gain_model_mc': tuple(['cmt_run_id', cmt_run_id, *cmt_options['gain_model']])})
+                st.config['fax_config_override_from_cmt'][fax_field] = (
+                         cmt_name + '_constant', fax_config[fax_field])
 
-    # User customized overwrite
-    cmt_options = straxen.get_corrections.get_cmt_options(st)  # This includes gain_model_mc
-    for option in cmt_option_overwrite:
+    # And as the last step - manual overrrides, since they have the highest priority
+    # User customized for simulation
+    for option in cmt_option_overwrite_sim:
         if option not in cmt_options:
             raise ValueError(f'Overwrite option {option} is not using CMT by default '
                              'you should just use set config')
-        _name_index = 2 if 'cmt_run_id' in cmt_options[option] else 0
-        st.config[option] = (cmt_options[option][_name_index] + '_constant', cmt_option_overwrite[option])
-
+        if option not in _config_overlap.values():
+            raise ValueError(f'Overwrite option {option} does not have mapping from '
+                             f'CMT to fax config!')
+        for fax_key, cmt_key in _config_overlap.items():
+            if cmt_key == option:
+                cmt_name = cmt_options_full[option]['correction']
+                st.config['fax_config_override_from_cmt'][fax_key] = (
+                                            cmt_name + '_constant',
+                                            cmt_option_overwrite_sim[option])
+            del(fax_key, cmt_key)
+    # User customized for simulation
+    for option in cmt_option_overwrite_proc:
+        if option not in cmt_options:
+            raise ValueError(f'Overwrite option {option} is not using CMT by default '
+                             'you should just use set config')
+        
+        if isinstance(cmt_options[option], str):
+            # URLConfig options can just be set to constants, no hacks needed
+            # But for now lets keep things consistent for people
+            st.config[option] = cmt_option_overwrite_proc[option]
+        else:
+            # CMT name allowed to be different from the config name
+            # WFSim needs the cmt name
+            cmt_name = cmt_options_full[option]['correction']
+            st.config[option] = (cmt_name + '_constant', 
+                                 cmt_option_overwrite_proc[option])
     # Only for simulations
     st.set_config({"event_info_function": "disabled"})
 
     return st
 
-
 ##
 # XENON1T
 ##
 
-
 x1t_context_config = {
     **common_opts,
     **dict(
@@ -369,16 +487,22 @@ x1t_common_config = dict(
     # Peaks
     # Smaller right extension since we applied the filter
     peak_right_extension=30,
-    s1_max_rise_time=60,
     s1_max_rise_time_post100=150,
     s1_min_coincidence=3,
     # Events*
     left_event_extension=int(0.3e6),
     right_event_extension=int(1e6),
-    elife_conf=('elife_xenon1t', 'v1', False),
+    elife=1e6,
     electron_drift_velocity=("electron_drift_velocity_constant", 1.3325e-4),
     max_drift_length=96.9,
     electron_drift_time_gate=("electron_drift_time_gate_constant", 1700),
+    se_gain=28.2,
+    avg_se_gain=28.2,
+    rel_extraction_eff=1.0,
+    s1_xyz_map=f'itp_map://resource://{pax_file("XENON1T_s1_xyz_lce_true_kr83m_SR1_pax-680_fdc-3d_v0.json")}?fmt=json',  # noqa
+    s2_xy_map=f'itp_map://resource://{pax_file("XENON1T_s2_xy_ly_SR1_v2.2.json")}?fmt=json',
+    g1=0.1426,
+    g2=11.55/(1 - 0.63),
 )
 
 
@@ -400,8 +524,13 @@ def demo():
     st.set_config(dict(
         hev_gain_model=('1T_to_pe_placeholder', False),
         gain_model=('1T_to_pe_placeholder', False),
-        elife_conf=('elife_constant', 1e6),
+        elife=1e6,
         electron_drift_velocity=("electron_drift_velocity_constant", 1.3325e-4),
+        se_gain=28.2,
+        avg_se_gain=28.2,
+        rel_extraction_eff=1.0,
+        s1_xyz_map=f'itp_map://resource://{pax_file("XENON1T_s1_xyz_lce_true_kr83m_SR1_pax-680_fdc-3d_v0.json")}?fmt=json',
+        s2_xy_map=f'itp_map://resource://{pax_file("XENON1T_s2_xy_ly_SR1_v2.2.json")}?fmt=json',
         ))
     return st
 
@@ -453,7 +582,10 @@ def xenon1t_dali(output_folder='./strax_data', build_lowlevel=False, **kwargs):
 
 def xenon1t_led(**kwargs):
     st = xenon1t_dali(**kwargs)
-    st.context_config['check_available'] = ('raw_records', 'led_calibration')
+    st.set_context_config(
+        {'check_available': ('raw_records', 'led_calibration'),
+         'free_options': list(x1t_context_config.keys())
+         })
     # Return a new context with only raw_records and led_calibration registered
     st = st.new_context(
         replace=True,
@@ -474,4 +606,5 @@ def xenon1t_simulation(output_folder='./strax_data'):
             **x1t_common_config),
         **x1t_context_config)
     st.register(wfsim.RawRecordsFromFax1T)
+    st.deregister_plugins_with_missing_dependencies()
     return st
