@@ -1,7 +1,6 @@
 """Return corrections from corrections DB
 """
 import warnings
-
 import pytz
 import numpy as np
 from functools import lru_cache
@@ -9,31 +8,36 @@ import strax
 import utilix
 import straxen
 import os
-from immutabledict import immutabledict
-
+from urllib.parse import urlparse, parse_qs
 
 export, __all__ = strax.exporter()
 
-corrections_w_file = ['mlp_model', 'gcn_model', 'cnn_model',
-                      's2_xy_map', 's1_xyz_map_mlp', 's1_xyz_map_cnn',
-                      's1_xyz_map_gcn', 'fdc_map_mlp', 'fdc_map_gcn',
-                      'fdc_map_cnn']
+corrections_w_file = ['mlp_model', 'cnn_model', 'gcn_model',
+                      's2_xy_map_mlp', 's2_xy_map_cnn', 's2_xy_map_gcn', 's2_xy_map',
+                      's1_xyz_map_mlp', 's1_xyz_map_cnn', 's1_xyz_map_gcn',
+                      'fdc_map_mlp', 'fdc_map_cnn', 'fdc_map_gcn']
 
 single_value_corrections = ['elife_xenon1t', 'elife', 'baseline_samples_nv',
-                            'electron_drift_velocity', 'electron_drift_time_gate']
+                            'electron_drift_velocity', 'electron_drift_time_gate',
+                            'se_gain', 'rel_extraction_eff']
 
 arrays_corrections = ['hit_thresholds_tpc', 'hit_thresholds_he',
                       'hit_thresholds_nv', 'hit_thresholds_mv']
 
 # needed because we pass these names as strax options which then get paired with the default reconstruction algorithm
 # important for apply_cmt_version
-posrec_corrections_basenames = ['s1_xyz_map', 'fdc_map']
+posrec_corrections_basenames = ['s1_xyz_map', 'fdc_map', 's2_xy_map']
 
 
+@export
 class CMTVersionError(Exception):
     pass
 
 
+class CMTnanValueError(Exception):
+    pass
+
+
 @export
 class CorrectionsManagementServices():
     """
@@ -104,7 +108,6 @@ class CorrectionsManagementServices():
                              f"available {single_value_corrections}, {arrays_corrections} and "
                              f"{corrections_w_file} ")
 
-    # TODO add option to extract 'when'. Also, the start time might not be the best
     # entry for e.g. for super runs
     # cache results, this would help when looking at the same gains
     @lru_cache(maxsize=None)
@@ -130,14 +133,22 @@ class CorrectionsManagementServices():
                 pmts = list(gains.keys())
                 for it_correction in pmts: # loop over all PMTs
                     if correction in it_correction:
-                        df = self.interface.read(it_correction)
+                        df = self.interface.read_at(it_correction, when)
+                        if df[version].isnull().values.any():
+                            raise CMTnanValueError(f"For {it_correction} there are NaN values, this means no correction available "
+                                                   f"for {run_id} in version {version}, please check e-logbook for more info ")
+ 
                         if version in 'ONLINE':
                             df = self.interface.interpolate(df, when, how='fill')
                         else:
                             df = self.interface.interpolate(df, when)
                         values.append(df.loc[df.index == when, version].values[0])
             else:
-                df = self.interface.read(correction)
+                df = self.interface.read_at(correction, when)
+                if df[version].isnull().values.any():
+                    raise CMTnanValueError(f"For {correction} there are NaN values, this means no correction available "
+                                           f"for {run_id} in version {version}, please check e-logbook for more info ")
+ 
                 if correction in corrections_w_file or correction in arrays_corrections or version in 'ONLINE':
                     df = self.interface.interpolate(df, when, how='fill')
                 else:
@@ -189,12 +200,10 @@ class CorrectionsManagementServices():
                 to_pe = self._get_correction(run_id, target_detector, version)
 
             # be cautious with very early runs, check that not all are None
-            if np.isnan(to_pe).all():
+            if np.isnan(to_pe).any():
                 raise ValueError(
                     f"to_pe(PMT gains) values are NaN, no data available "
-                    f"for {run_id} in the gain model with version "
-                    f"{version}, please set constant values for "
-                    f"{run_id}")
+                    f"for {run_id} in the gain model with version")
 
         else:
             raise ValueError(f"{model_type} not implemented for to_pe values")
@@ -241,7 +250,6 @@ class CorrectionsManagementServices():
                              f"Please contact CMT manager and yell at him")
         return file_name
 
-    # TODO change to st.estimate_start_time
     def get_start_time(self, run_id):
         """
         Smart logic to return start time from runsDB
@@ -305,14 +313,13 @@ def cacheable_naming(*args, fmt='.npy', base='./resource_cache/'):
         except (FileExistsError, PermissionError):
             pass
     for arg in args:
-        if not type(arg) == str:
+        if not isinstance(arg, str):
             raise TypeError(f'One or more args of {args} are not strings')
     return base + '_'.join(args) + fmt
 
 
 class GainsNotFoundError(Exception):
     """Fatal error if a None value is returned by the corrections"""
-    pass
 
 
 def get_cmt_local_versions(global_version):
@@ -320,8 +327,13 @@ def get_cmt_local_versions(global_version):
     return cmt.get_local_versions(global_version)
 
 
+def args_idx(x):
+    """Get the idx of "?" in the string"""
+    return x.rfind('?') if '?' in x else None
+
+
 @strax.Context.add_method
-def apply_cmt_version(context: strax.Context, cmt_global_version: str):
+def apply_cmt_version(context: strax.Context, cmt_global_version: str) -> None:
     """Sets all the relevant correction variables
     :param cmt_global_version: A specific CMT global version, or 'latest' to get the newest one
     :returns None
@@ -343,20 +355,53 @@ def apply_cmt_version(context: strax.Context, cmt_global_version: str):
     # we want this error to occur in order to keep fixed global versions
     cmt_config = dict()
     failed_keys = []
-    for option, tup in cmt_options.items():
-        try:
-            # might need to modify correction name to include position reconstruction algo
-            correction_name = tup[0]
-            if correction_name in posrec_corrections_basenames:
-                correction_name += f"_{posrec_algo}"
-            new_tup = (tup[0], local_versions[correction_name], tup[2])
-        except KeyError:
-            failed_keys.append(option)
+
+    for option, option_info in cmt_options.items():
+        # name of the CMT correction, this is not always equal to the strax option
+        correction_name = option_info['correction']
+        # actual config option
+        # this could be either a CMT tuple or a URLConfig
+        value = option_info['strax_option']
+
+        # might need to modify correction name to include position reconstruction algo
+        # this is a bit of a mess, but posrec configs are treated differently in the tuples
+        # URL configs should already include the posrec suffix
+        # (it's real mess -- we should drop tuple configs)
+        if correction_name in posrec_corrections_basenames:
+            correction_name += f"_{posrec_algo}"
+
+        # now see if our correction is in our local_versions dict
+        if correction_name in local_versions:
+            if isinstance(value, str) and 'cmt://' in value:
+                new_value = replace_url_version(value, local_versions[correction_name])
+            # if it is a tuple, make a new tuple
+            else:
+                new_value = (value[0], local_versions[correction_name], value[2])
+        else:
+            if correction_name not in failed_keys:
+                failed_keys.append(correction_name)
             continue
-        cmt_config[option] = new_tup
+
+        cmt_config[option] = new_value
+
     if len(failed_keys):
         failed_keys = ', '.join(failed_keys)
-        raise CMTVersionError(f"CMT version {cmt_global_version} is not compatible with this straxen version! "
-                              f"CMT {cmt_global_version} is missing these corrections: {failed_keys}")
+        msg = f"CMT version {cmt_global_version} is not compatible with this straxen version! " \
+              f"CMT {cmt_global_version} is missing these corrections: {failed_keys}"
+
+        # only raise a warning if we are working with the online context
+        if cmt_global_version == "global_ONLINE":
+            warnings.warn(msg, UserWarning)
+        else:
+            raise CMTVersionError(msg)
 
     context.set_config(cmt_config)
+
+
+def replace_url_version(url, version):
+    """Replace the local version of a correction in a CMT config"""
+    kwargs = {k: v[0] for k, v in parse_qs(urlparse(url).query).items()}
+    kwargs['version'] = version
+    args = [f"{k}={v}" for k, v in kwargs.items()]
+    args_str = "&".join(args)
+    return f'{url[:args_idx(url)]}?{args_str}'
