@@ -1,38 +1,51 @@
 import strax
 import straxen
+from straxen.get_corrections import get_correction_from_cmt
 import numpy as np
-from scipy.special import betainc, gammaln, loggamma
-import warnings
-export, __all__ = strax.exporter()
+import numba
+from straxen.numbafied_scipy import numba_gammaln, numba_betainc
+from scipy.special import loggamma
+import tarfile
+import tempfile
 
+export, __all__ = strax.exporter()
 
 @export
 @strax.takes_config(
-    strax.Option('s1_optical_map', help='S1 (x, y, z) optical/pattern map.',
+    strax.Option('s1_optical_map', help='S1 (x, y, z) optical/pattern map.', infer_type=False,
                  default='XENONnT_s1_xyz_patterns_LCE_corrected_qes_MCva43fa9b_wires.pkl'),
-    strax.Option('s2_optical_map', help='S2 (x, y) optical/pattern map.',
+    strax.Option('s2_optical_map', help='S2 (x, y) optical/pattern map.', infer_type=False,
                  default='XENONnT_s2_xy_patterns_LCE_corrected_qes_MCva43fa9b_wires.pkl'),
-    strax.Option('s1_aft_map', help='Date drive S1 area fraction top map.',
-                 default='s1_aft_dd_xyz_XENONnT_Kr83m_41500eV_25May2021.json'),
+    strax.Option('s2_tf_model', help='S2 (x, y) optical data-driven model', infer_type=False,
+                 default='XENONnT_s2_optical_map_data_driven_ML_v0_2021_11_25.tar.gz'),
+    strax.Option('s1_aft_map', help='Date drive S1 area fraction top map.', infer_type=False,
+                 default='s1_aft_dd_xyz_XENONnT_Kr83m_41500eV_31Oct2021.json'),
     strax.Option('mean_pe_per_photon', help='Mean of full VUV single photon response',
-                 default=1.2),
-    strax.Option('gain_model',
+                 default=1.2, infer_type=False,),
+    strax.Option('gain_model', infer_type=False,
                  help='PMT gain model. Specify as (model_type, model_config)'),
     strax.Option('n_tpc_pmts', type=int,
                  help='Number of TPC PMTs'),
     strax.Option('n_top_pmts', type=int,
                  help='Number of top TPC PMTs'),
-    strax.Option('s1_min_area_pattern_fit',
+    strax.Option('s1_min_area_pattern_fit', infer_type=False,
                  help='Skip EventPatternFit reconstruction if S1 area (PE) is less than this',
-                 default=3),
-    strax.Option('s2_min_area_pattern_fit',
+                 default=2),
+    strax.Option('s2_min_area_pattern_fit', infer_type=False,
                  help='Skip EventPatternFit reconstruction if S2 area (PE) is less than this',
                  default=10),
     strax.Option('store_per_channel', default=False, type=bool,
                  help='Store normalized LLH per channel for each peak'),
     strax.Option('max_r_pattern_fit', default=straxen.tpc_r, type=float,
                  help='Maximal radius of the peaks where llh calculation will be performed'),
+    strax.Option(name='electron_drift_velocity', infer_type=False,
+                 help='Vertical electron drift velocity in cm/ns (1e4 m/ms)',
+                 default=("electron_drift_velocity", "ONLINE", True)),
+    strax.Option(name='electron_drift_time_gate', infer_type=False,
+                 help='Electron drift time from the gate in ns',
+                 default=("electron_drift_time_gate", "ONLINE", True)),
 )
+
 class EventPatternFit(strax.Plugin):
     '''
     Plugin that provides patter information for events
@@ -40,13 +53,17 @@ class EventPatternFit(strax.Plugin):
     
     depends_on = ('event_area_per_channel', 'event_basics', 'event_positions')
     provides = 'event_pattern_fit'
-    __version__ = '0.0.7'
+    __version__ = '0.1.1'
 
     def infer_dtype(self):
         dtype = [('s2_2llh', np.float32,
                   'Modified Poisson likelihood value for main S2 in the event'),
+                 ('s2_neural_2llh', np.float32,
+                  'Data-driven based likelihood value for main S2 in the event'),
                  ('alt_s2_2llh', np.float32,
                   'Modified Poisson likelihood value for alternative S2'),
+                 ('alt_s2_neural_2llh', np.float32,
+                  'Data-driven based likelihood value for alternative S2 in the event'),
                  ('s1_2llh', np.float32,
                   'Modified Poisson likelihood value for main S1'),
                  ('s1_top_2llh', np.float32,
@@ -89,6 +106,8 @@ class EventPatternFit(strax.Plugin):
         return dtype
     
     def setup(self):
+        self.electron_drift_velocity = get_correction_from_cmt(self.run_id, self.config['electron_drift_velocity'])
+        self.electron_drift_time_gate = get_correction_from_cmt(self.run_id, self.config['electron_drift_time_gate'])
         self.mean_pe_photon = self.config['mean_pe_per_photon']
         
         # Getting S1 AFT maps
@@ -106,6 +125,21 @@ class EventPatternFit(strax.Plugin):
             straxen.get_resource(
                 self.config['s2_optical_map'],
                 fmt=self._infer_map_format(self.config['s2_optical_map'])))
+
+        # Getting S2 data-driven tensorflow models
+        downloader = straxen.MongoDownloader()
+        self.model_file = downloader.download_single(self.config['s2_tf_model'])
+        with tempfile.TemporaryDirectory() as tmpdirname:
+            tar = tarfile.open(self.model_file, mode="r:gz")
+            tar.extractall(path=tmpdirname)
+
+            import tensorflow as tf
+            def _logl_loss(patterns_true, likelihood):
+                return likelihood / 10.
+            self.model = tf.keras.models.load_model(tmpdirname,
+                                                    custom_objects={"_logl_loss": _logl_loss})
+            self.model_chi2 = tf.keras.Model(self.model.inputs,
+                                             self.model.get_layer('Likelihood').output)
         
         # Getting gain model to get dead PMTs
         self.to_pe = straxen.get_correction_from_cmt(self.run_id, self.config['gain_model'])
@@ -125,12 +159,19 @@ class EventPatternFit(strax.Plugin):
         
         # Computing LLH values for S2s
         self.compute_s2_llhvalue(events, result)
-        
+
+        # Computing chi2 values for S2s
+        self.compute_s2_neural_llhvalue(events, result)
+
         # Computing binomial test for s1 area fraction top
-        s1_area_fraction_top_probability = np.vectorize(_s1_area_fraction_top_probability)
         positions = np.vstack([events['x'], events['y'], events['z']]).T
         aft_prob = self.s1_aft_map(positions)
         
+        alt_s1_interaction_drift_time = events['s2_center_time']-events['alt_s1_center_time']
+        alt_s1_interaction_z = -self.electron_drift_velocity*(alt_s1_interaction_drift_time-self.electron_drift_time_gate)
+        alt_positions = np.vstack([events['x'], events['y'], alt_s1_interaction_z]).T     
+        alt_aft_prob = self.s1_aft_map(alt_positions)
+        
         # main s1 events
         mask_s1 = ~np.isnan(aft_prob)
         mask_s1 &= ~np.isnan(events['s1_area'])
@@ -152,7 +193,7 @@ class EventPatternFit(strax.Plugin):
             result['s1_photon_fraction_top_discrete_probability'][mask_s1] = s1_area_fraction_top_probability(*arg, 'discrete')
         
         # alternative s1 events
-        mask_alt_s1 = ~np.isnan(aft_prob)
+        mask_alt_s1 = ~np.isnan(alt_aft_prob)
         mask_alt_s1 &= ~np.isnan(events['alt_s1_area'])
         mask_alt_s1 &= ~np.isnan(events['alt_s1_area_fraction_top'])
         
@@ -247,23 +288,22 @@ class EventPatternFit(strax.Plugin):
             # - must have total area larger minimal one
             # - must have positive AFT
             x, y = events[t_+'_x'], events[t_+'_y']
-            cur_s2_bool = (events[t_+'_area']>self.config['s2_min_area_pattern_fit'])
-            cur_s2_bool &= (events[t_+'_index']!=-1)
-            cur_s2_bool &= (events[t_+'_area_fraction_top']>0)
-            cur_s2_bool &= (x**2 + y**2) < self.config['max_r_pattern_fit']**2
+            s2_mask = (events[t_+'_area']>self.config['s2_min_area_pattern_fit'])
+            s2_mask &= (events[t_+'_area_fraction_top']>0)
+            s2_mask &= (x**2 + y**2) < self.config['max_r_pattern_fit']**2
             
             # default value is nan, it will be ovewrite if the event satisfy the requirments
             result[t_+'_2llh'][:] = np.nan
             
             # Making expectation patterns [ in PE ]
-            if np.sum(cur_s2_bool):
-                s2_map_effs = self.s2_pattern_map(np.array([x, y]).T)[cur_s2_bool, 0:self.config['n_top_pmts']]
+            if np.sum(s2_mask):
+                s2_map_effs = self.s2_pattern_map(np.array([x, y]).T)[s2_mask, 0:self.config['n_top_pmts']]
                 s2_map_effs = s2_map_effs[:, self.pmtbool_top]
-                s2_top_area = (events[t_+'_area_fraction_top']*events[t_+'_area'])[cur_s2_bool]
+                s2_top_area = (events[t_+'_area_fraction_top']*events[t_+'_area'])[s2_mask]
                 s2_pattern  = s2_top_area[:, None]*s2_map_effs/np.sum(s2_map_effs, axis=1)[:,None]
 
                 # Getting pattern from data
-                s2_top_area_per_channel = events[t_+'_area_per_channel'][cur_s2_bool, 0:self.config['n_top_pmts']]
+                s2_top_area_per_channel = events[t_+'_area_per_channel'][s2_mask, 0:self.config['n_top_pmts']]
                 s2_top_area_per_channel = s2_top_area_per_channel[:, self.pmtbool_top]
 
                 # Calculating LLH, this is shifted Poisson
@@ -279,17 +319,33 @@ class EventPatternFit(strax.Plugin):
                                      areas = s2_top_area_per_channel, 
                                      mean_pe_photon=self.mean_pe_photon)
                                )
-                result[t_+'_2llh'][cur_s2_bool] = np.sum(norm_llh_val, axis=1)
+                result[t_+'_2llh'][s2_mask] = np.sum(norm_llh_val, axis=1)
 
                 if self.config['store_per_channel']:
                     store_patterns = np.zeros((s2_pattern.shape[0], self.config['n_top_pmts']) )
                     store_patterns[:, self.pmtbool_top] = s2_pattern
-                    result[t_+'_pattern'][cur_s2_bool] = store_patterns#:s2_pattern[cur_s2_bool]
+                    result[t_+'_pattern'][s2_mask] = store_patterns#:s2_pattern[s2_mask]
 
                     store_2LLH_ch = np.zeros((norm_llh_val.shape[0], self.config['n_top_pmts']) )
                     store_2LLH_ch[:, self.pmtbool_top] = norm_llh_val
-                    result[t_+'_2llh_per_channel'][cur_s2_bool] = store_2LLH_ch
-                    
+                    result[t_+'_2llh_per_channel'][s2_mask] = store_2LLH_ch
+
+    def compute_s2_neural_llhvalue(self, events, result):
+        for t_ in ['s2', 'alt_s2']:
+            x, y = events[t_ + '_x'], events[t_ + '_y']
+            s2_mask = (events[t_ + '_area'] > self.config['s2_min_area_pattern_fit'])
+            s2_mask &= (events[t_ + '_area_fraction_top'] > 0)
+
+            # default value is nan, it will be ovewrite if the event satisfy the requirements
+            result[t_ + '_neural_2llh'][:] = np.nan
+
+            # Produce position and top pattern to feed tensorflow model, return chi2/N
+            if np.sum(s2_mask):
+                s2_pos = np.stack((x, y)).T[s2_mask]
+                s2_pat = events[t_ + '_area_per_channel'][s2_mask, 0:self.config['n_top_pmts']]
+                # Output[0]: loss function, -2*log-likelihood, Output[1]: chi2
+                result[t_ + '_neural_2llh'][s2_mask] = self.model_chi2.predict({'xx': s2_pos, 'yy': s2_pat})[1]
+
     @staticmethod
     def _infer_map_format(map_name, known_formats=('pkl', 'json', 'json.gz')):
         for fmt in known_formats:
@@ -307,142 +363,140 @@ def neg2llh_modpoisson(mu=None, areas=None, mean_pe_photon=1.0):
     mean_pe_photon - mean of area responce for one photon
     """
     with np.errstate(divide='ignore', invalid='ignore'):
-        res = 2.*(mu - 
-                  (areas/mean_pe_photon)*np.log(mu) + 
-                  loggamma((areas/mean_pe_photon)+1) + 
+        fraction = areas/mean_pe_photon
+        res = 2.*(mu -
+                  (fraction)*np.log(mu) +
+                  loggamma((fraction)+1) +
                   np.log(mean_pe_photon)
                  )
-    is_zero = ~(areas>0)    # If area equals or smaller than 0 - assume 0
+    is_zero = areas <= 0    # If area equals or smaller than 0 - assume 0
     res[is_zero] = 2.*mu[is_zero]
     # if zero channel has negative expectation, assume LLH to be 0 there
     # this happens in the normalization factor calculation when mu is received from area
-    neg_mu = mu<0.0
-    res[is_zero*neg_mu] = 0.0
+    neg_mu = mu < 0.0
+    res[is_zero | neg_mu] = 0.0
     return res
 
+
 # continuous and discrete binomial test
-# https://github.com/poliastro/cephes/blob/master/src/bdtr.c
 
-def bdtrc(k, n, p):
-    if (k < 0):
-        return (1.0)
-    if (k == n):
-        return (0.0)
-    dn = n - k
-    if (k == 0):
-        if (p < .01):
-            dk = -np.expm1(dn * np.log1p(-p))
-        else:
-            dk = 1.0 - np.exp(dn * np.log(1.0 - p))
-    else:
-        dk = k + 1
-        dk = betainc(dk, dn, p)
-    return dk
-
-def bdtr(k, n, p):
-    if (k < 0):
-        return np.nan
-    if (k == n):
-        return (1.0)
-    dn = n - k
-    if (k == 0):
-        dk = np.exp(dn * np.log(1.0 - p))
-    else:
-        dk = k + 1
-        dk = betainc(dn, dk, 1.0 - p)
-    return dk
-
-# continuous binomial distribution
-def binom_pmf(k, n, p):
-    scale_log = gammaln(n + 1) - gammaln(n - k + 1) - gammaln(k + 1)
+@numba.njit
+def lbinom_pmf(k, n, p):
+    """Log of binomial probability mass function approximated with gamma function"""
+    scale_log = numba_gammaln(n + 1) - numba_gammaln(n - k + 1) - numba_gammaln(k + 1)
     ret_log = scale_log + k * np.log(p) + (n - k) * np.log(1 - p)
-    return np.exp(ret_log)
+    return ret_log
 
+
+@numba.njit
+def binom_pmf(k, n, p):
+    """Binomial probability mass function approximated with gamma function"""
+    return np.exp(lbinom_pmf(k, n, p))
+
+
+@numba.njit
 def binom_cdf(k, n, p):
-    return bdtr(k, n, p)
+    if k >= n:
+        return 1.0
+    return numba_betainc(n - k, k + 1, 1.0 - p)
 
+
+@numba.njit
 def binom_sf(k, n, p):
-    return bdtrc(k, n, p)
+    return 1 - binom_cdf(k, n, p)
 
+
+@numba.njit
+def lbinom_pmf_diriv(k, n, p, dk=1e-7):
+    """Numerical dirivitive of Binomial pmf approximated with gamma function"""
+    if k + dk < n:
+        return (lbinom_pmf(k + dk, n, p) - lbinom_pmf(k, n, p)) / dk
+    else:
+        return (lbinom_pmf(k - dk, n, p) - lbinom_pmf(k, n, p)) / - dk
+
+
+@numba.njit(cache=True)
+def _numeric_derivative(y0, y1, err, target, x_min, x_max, x0, x1):
+    """Get close to <target> by doing a numeric derivative"""
+    if abs(y1 - y0) < err:
+        # break by passing dx == 0
+        return 0., x1, x1
+
+    x = (target - y0) / (y1 - y0) * (x1 - x0) + x0
+    x = min(x, x_max)
+    x = max(x, x_min)
+
+    dx = abs(x - x1)
+    x0 = x1
+    x1 = x
+
+    return dx, x0, x1
+
+
+@numba.njit
+def lbinom_pmf_mode(x_min, x_max, target, args, err=1e-7, max_iter=50):
+    """Find the root of the derivative of log Binomial pmf with secant method"""
+    x0 = x_min
+    x1 = x_max
+    dx = abs(x1 - x0)
+
+    while (dx > err) and (max_iter > 0):
+        y0 = lbinom_pmf_diriv(x0, *args)
+        y1 = lbinom_pmf_diriv(x1, *args)
+        dx, x0, x1 = _numeric_derivative(y0, y1, err, target, x_min, x_max, x0, x1)
+        max_iter -= 1
+    return x1
+
+
+@numba.njit
+def lbinom_pmf_inverse(x_min, x_max, target, args, err=1e-7, max_iter=50):
+    """Find the where the log Binomial pmf cross target with secant method"""
+    x0 = x_min
+    x1 = x_max
+    dx = abs(x1 - x0)
+
+    while (dx > err) and (max_iter > 0):
+        y0 = lbinom_pmf(x0, *args)
+        y1 = lbinom_pmf(x1, *args)
+        dx, x0, x1 = _numeric_derivative(y0, y1, err, target, x_min, x_max, x0, x1)
+        max_iter -= 1
+    return x1
+
+
+@numba.njit
 def binom_test(k, n, p):
     """
     The main purpose of this algorithm is to find the value j on the
-    other side of the mean that has the same probability as k, and
+    other side of the mode that has the same probability as k, and
     integrate the tails outward from k and j. In the case where either
     k or j are zero, only the non-zero tail is integrated.
     """
-    d = binom_pmf(k, n, p)
-    rerr = 1 + 1e-7
-    d = d * rerr
-    # define number of interaction for finding the the value j
-    # the exceptional case of n<=0, is avoid since n_iter is at least 2
-    if n > 0:
-        n_iter = int(max(np.round(np.log10(n)) + 1, 2))
-    else:
-        n_iter = 2
+    mode = lbinom_pmf_mode(0, n, 0, (n, p))
 
-    if k < n * p:
-        # if binom_pmf(n, n, p) > d, with d<<1e-3, means that we have
-        # to look for j value above n. It is likely that the binomial
-        # test for such j is extremely low such that we can stop
-        # the algorithm and return 0
-        if binom_pmf(n, n, p) > d:
-            for n_ in np.arange(n, 2*n, 1):
-                if binom_pmf(n_, n, p) < d:
-                    j_min, j_max = k, n_
-                    do_test = True
-                    break
-                do_test = False
-        else:
-            j_min, j_max = k, n
-            do_test = True
-        def _check_(d, y0, y1):
-            return (d>y1) and (d<=y0)
+    if k <= mode:
+        j_min, j_max = mode, n
     else:
-        if binom_pmf(0, n, p) > d:
-            n_iter, j_min, j_max = 0, 0, 0
-        else:
-            j_min, j_max = 0, k
-        do_test = True
-        def _check_(d, y0, y1):
-            return (d>=y0) and (d<y1)
+        j_min, j_max = 0, mode
 
-    # if B(k;n,p) is already 0 or I can't find the j in the other side of the mean
-    # the returned binomial test is 0
-    if (d==0)|(not do_test):
-        pval = 0.0
-    else:
-        # Here we are actually looking for j
-        for i in range(n_iter):
-            n_pts = int(j_max - j_min)
-            if (i<2) and (n_pts < 50): 
-                n_pts = 50
-            j_range = np.linspace(j_min, j_max, n_pts, endpoint=True)
-            y = binom_pmf(j_range, n, p)
-            for i in range(len(j_range) - 1):
-                if _check_(d, y[i], y[i + 1]):
-                    j_min, j_max = j_range[i], j_range[i + 1]
-                    break
-        j = max(min((j_min + j_max) / 2, n), 0)
+    target = lbinom_pmf(k, n, p)
+    j = lbinom_pmf_inverse(j_min, j_max, target, (n, p))
+
+    pval = 0
+    if min(k, j) > 0:
+        pval += binom_cdf(min(k, j), n, p)
+    if max(k, j) > 0:
+        pval += binom_sf(max(k, j), n, p)
+    pval = min(1.0, pval)
 
-        # One side or two side
-        # binomial test
-        if k * j == 0:
-            pval = binom_sf(max(k, j), n, p)
-        else:
-            pval = binom_cdf(min(k, j), n, p) + binom_sf(max(k, j), n, p)
-        pval = min(1.0, pval)
-        
     return pval
 
 
-def _s1_area_fraction_top_probability(aft_prob, area_tot, area_fraction_top, mode='continuous'):
-    '''
-    Wrapper that does the S1 AFT probability calculation for you
-    '''
-    
+@np.vectorize
+@numba.njit
+def s1_area_fraction_top_probability(aft_prob, area_tot, area_fraction_top, mode='continuous'):
+    """Function to compute the S1 AFT probability"""
     area_top = area_tot * area_fraction_top
-    
+
     # Raise a warning in case one of these three condition is verified
     # and return binomial test equal to nan since they are not physical
     # k: size_top, n: size_tot, p: aft_prob
@@ -459,11 +513,11 @@ def _s1_area_fraction_top_probability(aft_prob, area_tot, area_fraction_top, mod
         # warnings.warn(f'k {area_top} must be >= 0')
         binomial_test = np.nan
         do_test = False
-        
+
     if do_test:
         if mode == 'discrete':
             binomial_test = binom_pmf(area_top, area_tot, aft_prob)
         else:
             binomial_test = binom_test(area_top, area_tot, aft_prob)
-        
+
     return binomial_test
