@@ -1,6 +1,6 @@
 import strax
 import numpy as np
-import numexpr
+from immutabledict import immutabledict
 
 export, __all__ = strax.exporter()
 
@@ -181,143 +181,122 @@ class OnlinePeakMonitor(strax.Plugin):
 @export
 @strax.takes_config(
     strax.Option(
-        'drift_time_vs_R2_cut_string',
-        type=str,
-        default='(s2_index > -1) & (s1_index > -1)',
-        help='Selection (like selection_str) applied to data for '
-             '"drift_time_vs_R2", cuts should be separated using "&"'),
+        'channel_map', 
+        track=False, 
+        type=immutabledict,
+        help='immutabledict mapping subdetector to (min, max) '
+             'channel number.'),
     strax.Option(
-        'drift_time_vs_R2_bounds',
-        type=tuple, default=((0, 5e3), (0, 3e6)),
-        help='Boundaries of histogram of drift time vs R^2.'),
+        'events_area_bounds',
+        type=tuple, default=(-0.5, 130.5),
+        help='Boundaries area histogram of events_nv_area_per_chunk [PE]'),
     strax.Option(
-        'online_event_monitor_nbins',
-        type=int, default=60,
-        help='Number of bins of histogram of online monitor. NB: these are 2D '
-             'histograms! Will be used for: '
-             'drift_time_vs_R2-histogram, '
-             'drift_time_vs_s2wdith-histogram, '
-             'drift_time_vs_s1aft-histogram, '
-             's1area_vs_s2area-histogram, '),
-    strax.Option(
-        'drift_time_vs_s2width_bounds',
-        type=tuple, default=((0, 3e6), (0, 1.5e4)),
-        help='Boundaries of histogram of drift time vs. s2width'),
-    strax.Option(
-        'drift_time_vs_s1aft_bounds',
-        type=tuple, default=((0, 3e6), (0, 1)),
-        help='Boundaries of histogram of drift time vs. s1aft'),
-    strax.Option(
-        's1area_vs_s2area_bounds',
-        type=tuple, default=((0, 5), (1, 7)),
-        help='Boundaries of the log-log histogram of s1 area vs. s2 area'),
+        'events_area_nbins',
+        type=int, default=131,
+        help='Number of bins of histogram of events_nv_area_per_chunk, '
+             'defined value 1 PE/bin')
 )
-class OnlineEventMonitor(strax.Plugin):
+class OnlineMonitorNV(strax.Plugin):
     """
-    Plugin to write data to the online-monitor. Data that is written by
-    this plugin should be small such as to not overload the runs-
-    database.
+    Plugin to write data of nVeto detector to the online-monitor. 
+    Data that is written by this plugin should be small (~MB/chunk) 
+    to not overload the runs-database.
 
-    This plugin takes 'event_basics'.
+    This plugin takes 'hitlets_nv' and 'events_nv'. Although they are
+    not strictly related, they are aggregated into a single data_type
+    in order to minimize the number of documents in the online monitor.
 
-    Produces 'online_event_monitor' with info on the events.
+    Produces 'online_monitor_nv' with info on the hitlets_nv and events_nv
     """
-    depends_on = ('event_basics',)
-    provides = 'online_event_monitor'
-    __version__ = '0.0.2'
-    data_kind = 'online_event_monitor'
+    depends_on = ('hitlets_nv', 'events_nv')
+    provides = 'online_monitor_nv'
+    data_kind = 'online_monitor_nv'
     rechunk_on_save = False
 
+    # Needed in case we make again an muVETO child.
+    ends_with = '_nv'
+
+    __version__ = '0.0.4'
+
     def infer_dtype(self):
-        n_bins = self.config['online_event_monitor_nbins']
-        bounds_drift_time_r2 = self.config['drift_time_vs_R2_bounds']
+        self.channel_range = self.config['channel_map']['nveto']
+        self.n_channel = (self.channel_range[1] - self.channel_range[0]) + 1
+        return veto_monitor_dtype(self.ends_with, self.n_channel, self.config['events_area_nbins'])
 
-        dtype = [
-            (('Start time of the chunk', 'time'),
-             np.int64),
-            (('End time of the chunk', 'endtime'),
-             np.int64),
-            (('Drift time vs R^2 histogram', 'drift_time_vs_R2_hist'),
-             (np.int64, (n_bins, n_bins))),
-            (('Drift time vs R^2 edges (linear space)', 'drift_time_vs_R2_bounds'),
-             (np.float64, np.shape(bounds_drift_time_r2))),
-            (('Drift time vs. s2width histogram', 'drift_time_vs_s2width_hist'),
-             (np.int64, (n_bins, n_bins))),
-            (('Drift time vs. s2width edges', 'drift_time_vs_s2width_bounds'),
-             (np.float64, np.shape(self.config['drift_time_vs_s2width_bounds']))),
-            (('Drift time vs. s1aft histogram', 'drift_time_vs_s1aft_hist'),
-             (np.int64, (n_bins, n_bins))),
-            (('Drift time vs. s1aft edges', 'drift_time_vs_s1aft_bounds'),
-             (np.float64, np.shape(self.config['drift_time_vs_s1aft_bounds']))),
-            (('S1 area vs. S2 area histogram', 's1area_vs_s2area_hist'),
-             (np.int64, (n_bins, n_bins))),
-            (('S1 area vs. S2 area edges (log-log)', 's1area_vs_s2area_bounds'),
-             (np.float64, np.shape(self.config['s1area_vs_s2area_bounds']))),
-        ]
-        return dtype
-
-    def compute(self, events, start, end):
+    def compute(self, hitlets_nv, events_nv, start, end):
         # General setup
         res = np.zeros(1, dtype=self.dtype)
         res['time'] = start
         res['endtime'] = end
 
-        # --- Drift time vs R^2 histogram ---
-        res['drift_time_vs_R2_bounds'] = self.config['drift_time_vs_R2_bounds']
+        # Count number of hitlets_nv per PMT
+        hitlets_channel_count, _ = np.histogram(hitlets_nv['channel'],
+                                                bins=self.n_channel,
+                                                range=[self.channel_range[0],
+                                                       self.channel_range[1] + 1])
+        res[f'hitlets{self.ends_with}_per_channel'] = hitlets_channel_count
 
-        dt_r2_events = strax.apply_selection(
-            events,
-            selection_str=self.config['drift_time_vs_R2_cut_string'])
-        res['drift_time_vs_R2_hist'] = self.drift_time_r2_hist(dt_r2_events)
+        # Count number of events_nv with coincidence cut
+        res[f'events{self.ends_with}_per_chunk'] = len(events_nv)
+        sel = events_nv['n_contributing_pmt'] >= 4
+        res[f'events{self.ends_with}_4coinc_per_chunk'] = np.sum(sel)
+        sel = events_nv['n_contributing_pmt'] >= 5
+        res[f'events{self.ends_with}_5coinc_per_chunk'] = np.sum(sel)
+        sel = events_nv['n_contributing_pmt'] >= 8
+        res[f'events{self.ends_with}_8coinc_per_chunk'] = np.sum(sel)
+        sel = events_nv['n_contributing_pmt'] >= 10
+        res[f'events{self.ends_with}_10coinc_per_chunk'] = np.sum(sel)
 
-        # drift time vs. s2 width histogram
-        res['drift_time_vs_s2width_bounds'] = self.config['drift_time_vs_s2width_bounds']
-        res['drift_time_vs_s2width_hist'] = self.drift_time_s2width_hist(events)
-
-        # drift time vs. s1 aft histogram
-        res['drift_time_vs_s1aft_bounds'] = self.config['drift_time_vs_s1aft_bounds']
-        res['drift_time_vs_s1aft_hist'] = self.drift_time_s1aft_hist(events)
-
-        # s1 area vs. s2 area (log-log)
-        res['s1area_vs_s2area_bounds'] = self.config['s1area_vs_s2area_bounds']
-        res['s1area_vs_s2area_hist'] = self.s1area_s2area_hist(events)
-
-        # Other event properties?
+        # Get histogram of events_nv_area per chunk
+        events_area, bins_ = np.histogram(events_nv['area'],
+                                          bins=self.config['events_area_nbins'],
+                                          range=self.config['events_area_bounds'])
+        res[f'events{self.ends_with}_area_per_chunk'] = events_area
         return res
 
-    def drift_time_r2_hist(self, data):
-        """Make drift time vs R^2 2D-hist"""
-        hist, _, _ = np.histogram2d(
-            (data['s2_x'] ** 2 + data['s2_y'] ** 2),
-            data['drift_time'],
-            range=self.config['drift_time_vs_R2_bounds'],
-            bins=self.config['online_event_monitor_nbins'])
-        return hist.T
 
-    def drift_time_s2width_hist(self, data):
-        """Make drift time vs. s2width 2D-hist"""
-        hist, _, _ = np.histogram2d(
-            data['drift_time'],
-            data['s2_range_50p_area'],
-            range=self.config['drift_time_vs_s2width_bounds'],
-            bins=self.config['online_event_monitor_nbins'])
-        return hist.T
+def veto_monitor_dtype(veto_name: str = '_nv',
+                       n_pmts: int = 120,
+                       n_bins: int = 131) -> list:
+    dtype = []
+    dtype += strax.time_fields  # because mutable
+    dtype += [((f'hitlets{veto_name} per channel', f'hitlets{veto_name}_per_channel'), (np.int64, n_pmts)),
+              ((f'events{veto_name}_area per chunk', f'events{veto_name}_area_per_chunk'), np.int64, n_bins),
+              ((f'events{veto_name} per chunk', f'events{veto_name}_per_chunk'), np.int64),
+              ((f'events{veto_name} 4-coincidence per chunk', f'events{veto_name}_4coinc_per_chunk'), np.int64),
+              ((f'events{veto_name} 5-coincidence per chunk', f'events{veto_name}_5coinc_per_chunk'), np.int64),
+              ((f'events{veto_name} 8-coincidence per chunk', f'events{veto_name}_8coinc_per_chunk'), np.int64),
+              ((f'events{veto_name} 10-coincidence per chunk', f'events{veto_name}_10coinc_per_chunk'), np.int64)
+             ]
+    return dtype
 
-    def drift_time_s1aft_hist(self, data):
-        """Make drift time vs. s1aft 2D-hist"""
-        hist, _, _ = np.histogram2d(
-            data['drift_time'],
-            data['s1_area_fraction_top'],
-            range=self.config['drift_time_vs_s1aft_bounds'],
-            bins=self.config['online_event_monitor_nbins'])
-        return hist.T
 
-    def s1area_s2area_hist(self, data):
-        """Make s1aft vs. drift time 2D-hist"""
-        hist, _, _ = np.histogram2d(
-            np.log10(data['s1_area']),
-            np.log10(data['s2_area']),
-            range=self.config['s1area_vs_s2area_bounds'],
-            bins=self.config['online_event_monitor_nbins']
-        )
-        return hist.T
+@export
+@strax.takes_config(
+    strax.Option(
+        'adc_to_pe_mv',
+        type=int, default=170.0,
+        help='conversion factor from ADC to PE for muon Veto')
+)
+class OnlineMonitorMV(OnlineMonitorNV):
+    __doc__ = OnlineMonitorNV.__doc__.replace('_nv', '_mv').replace('nVeto', 'muVeto')
+    depends_on = ('hitlets_mv', 'events_mv')
+    provides = 'online_monitor_mv'
+    data_kind = 'online_monitor_mv'
+    rechunk_on_save = False
+
+    # Needed in case we make again an muVETO child.
+    ends_with = '_mv'
+    child_plugin = True
+
+    __version__ = '0.0.1'
+
+    def infer_dtype(self):
+        self.channel_range = self.config['channel_map']['mv']
+        self.n_channel = (self.channel_range[1] - self.channel_range[0]) + 1
+        return veto_monitor_dtype(self.ends_with, self.n_channel, self.config['events_area_nbins'])
+
+    def compute(self, hitlets_mv, events_mv, start, end):
+        events_mv = np.copy(events_mv)
+        events_mv['area'] *= 1./self.config['adc_to_pe_mv']
+        return super().compute(hitlets_mv, events_mv, start, end)
